/*
 * Copyright (c) 2017, Peter Frank Perroni (pfperroni@gmail.com)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * Additionally, if used for any scientific purpose, the following reference
 * must be cited:
 * 
 * Peter Frank Perroni, Daniel Weingaertner, and Myriam Regattieri Delgado.
 * 2017. Estimating Stop Conditions of Swarm Based Stochastic Metaheuristic
 * Algorithms. In Proceedings of GECCO '17, Berlin, Germany, July 15-19, 2017,
 * pg 43-50. DOI: http://dx.doi.org/10.1145/3071178.3071326
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "CSMOn.hpp"

/**
 * @brief Class for CSMOn.
 * 
 * Always call this class directly (instead of the search method).
 *
 * @param search The instance for the search method.
 * @param M The maximum number of fitness function evaluations allocated for this run.
 * @param R The relaxation to be used for the limit calculations, in the interval ]0,1[. For decreasing relaxation, provide it in negative values.
 * @param minEstimatedFit The lowest fitness value expected.
 */
CSMOn::CSMOn(ISearch *search, int M, double R, double minEstimatedFit) {
	this->search = search;
	this->M = M;
	this->minEstimatedFit = minEstimatedFit;
	if(R < 0){
		autoRelaxation = true;
		this->R = abs(R);
	}
	else{
		autoRelaxation = false;
		this->R = R;
	}

	search->startup();

	s = -1;
}

CSMOn::~CSMOn() {}

ISearch::~ISearch() {}

/**
 * @brief Call this method to execute the search.
 */
void CSMOn::run() {
	int pT = -1, pS = -1;
	double r = (autoRelaxation ? 0.99 : R);
	getBest(1);
	do{
		r = (autoRelaxation ? max(r*r, R) : r);
		if(pS == -1)
			pT = adjustExp(r);
		if(pT > 0)
			pS = adjustLog(r, pT);
	}while(getNEvals() < M && (r > R || pS == -1));
}

int CSMOn::adjustExp(double r) {
	int sPrev = s;
	getBest(2);
	if(s-sPrev < 2) return -1;
	int pB = -1;
	double alpha1, alpha2;
	while(getNEvals() < M){
		if(decayE() < r && decayL() < r){
			if(pB == -1){
				pB = s-2;
				alpha2 = alphaE(pB, s);
			}
			else{
				alpha1 = alpha2;
				alpha2 = alphaE(pB, s);
				if(alpha2 < alpha1) return s;
			}
		}
		else pB = -1;
		getBest(1);
	}
	return -1;
}

int CSMOn::adjustLog(double r, int pT) {
	int sPrev = s;
	getBest(3);
	if(s-sPrev < 3) return -1;
	double alpha1 = alphaP(pT, s-1);
	double alpha2 = alphaP(pT, s);
	while(alpha2 >= alpha1 && getNEvals() < M){
		if(decayE() >= r || decayL() >= r) return -1;
		getBest(1);
		alpha1 = alpha2;
		alpha2 = alphaP(pT, s);
	}
	return s;
}

void CSMOn::getBest(int nBest) {
	for(int i=0; i < nBest && search->getNEvals() < M; i++){
		search->next(M);
		gb.push_back(t_point(search->getNEvals(), search->getFitness()));
		s++;
	}
}

double CSMOn::decayE() {
	return fabs(1 - (gb[s].y-minEstimatedFit) / (gb[s-1].y-minEstimatedFit));
}

double CSMOn::decayL() {
	return fabs(1 - (gb[s].y-gb[s-1].y) / (gb[s-1].y-gb[s-2].y));
}

double CSMOn::alphaE(int p1, int p2) {
	int i, n = p2 - p1 + 1;
	double yAvg = 0, yAvgLn, ySumLn = 0, xSum = 0, xAvg, S1 = 0, S2 = 0, aux;
	for(i=p1; i <= p2; i++){
		xSum += gb[i].x;
		yAvg += gb[i].y;
		ySumLn += log(gb[i].y);
	}
	xAvg = xSum / n;
	yAvg /= n;
	yAvgLn = ySumLn / n;
	for(i=p1; i <= p2; i++){
		aux = (gb[i].x - xAvg);
		S1 += aux * (gb[i].y - yAvgLn);
		S2 += aux * aux;
	}
	return (ySumLn - (S1 / S2) * xSum) / n;
}

double CSMOn::alphaP(int p1, int p2) {
	int i, n = p2 - p1 + 1;
	double yAvgLog, ySumLog = 0, xAvgLog, xSumLog = 0, S1 = 0, S2 = 0, aux;
	for(i=p1; i <= p2; i++){
		xSumLog += log10(gb[i].x);
		ySumLog += log10(gb[i].y);
	}
	xAvgLog = xSumLog / n;
	yAvgLog = ySumLog / n;
	for(i=p1; i <= p2; i++){
		aux = (log10(gb[i].x) - xAvgLog);
		S1 += aux * (log10(gb[i].y) - yAvgLog);
		S2 += aux * aux;
	}
	return (ySumLog - (S1 / S2) * xSumLog) / n;
}

/**
 * @brief Get the final fitness value.
 *
 *
 * @return The final fitness value.
 */
double CSMOn::getFitness() {
	return search->getFitness();
}

/**
 * @brief Get the actual number of evaluations executed.
 *
 *
 * @return The actual number of evaluations executed.
 */
int CSMOn::getNEvals() {
	return search->getNEvals();
}

/**
 * @brief Get the final optimized result (position).
 *
 *
 * @return The final position.
 */
int CSMOn::getBestPos(double *x) {
	return search->getBestPos(x);
}
