/*
 * Copyright (c) 2017, Peter Frank Perroni (pfperroni@gmail.com)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * Additionally, if used for any scientific purpose, the following reference
 * must be cited:
 * 
 * Peter Frank Perroni, Daniel Weingaertner, and Myriam Regattieri Delgado.
 * 2017. Estimating Stop Conditions of Swarm Based Stochastic Metaheuristic
 * Algorithms. In Proceedings of GECCO '17, Berlin, Germany, July 15-19, 2017,
 * pg 43-50. DOI: http://dx.doi.org/10.1145/3071178.3071326
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef ISEARCH_HPP_
#define ISEARCH_HPP_

/**
 * @brief The interface that the search methods must to implement.
 * 
 * @date 04/Mar/2017
 * @author Peter Frank Perroni (pfperroni@gmail.com)
 */
class ISearch{
public:
	ISearch(){}
	virtual ~ISearch()=0;
	/**
	 * @brief Startup the search method.
	 */
	virtual void startup()=0;
	/**
	 * @brief Obtain the next improvement.
	 *
	 * @param M The maximum number of evaluations allowed.
	 */
	virtual void next(int M)=0;
	/**
	 * @brief Get the best result obtained up to the moment.
	 *
	 * @param _x A pointer to store the positions of the best result.
	 * @return An ID for the best result (implementation specific).
	 */
	virtual int getBestPos(double *_x)=0;
	/**
	 * @brief Get the number of fitness function evaluations performed up to the moment.
	 *
	 * @return The number of evaluations performed.
	 */
	virtual int getNEvals()=0;
	/**
	 * @brief Get the best fitness value found up to the moment.
	 *
	 *
	 * @return The best fitness value found.
	 */
	virtual double getFitness()=0;
};

#endif /* ISEARCH_HPP_ */
