/*
 * Copyright (c) 2017, Peter Frank Perroni (pfperroni@gmail.com)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * Additionally, if used for any scientific purpose, the following reference
 * must be cited:
 * 
 * Peter Frank Perroni, Daniel Weingaertner, and Myriam Regattieri Delgado.
 * 2017. Estimating Stop Conditions of Swarm Based Stochastic Metaheuristic
 * Algorithms. In Proceedings of GECCO '17, Berlin, Germany, July 15-19, 2017,
 * pg 43-50. DOI: http://dx.doi.org/10.1145/3071178.3071326
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "PSO.hpp"

/**
 * @brief A standard implementation of PSO.
 *
 * @param fitnessFunction The callback reference to the fitness function.
 * @param s1 The lower bound of the search space.
 * @param s2 The upper bound of the search space.
 * @param p The number of particles.
 * @param n The number of dimensions of the problem.
 * @param w The acceleration coefficient. For linear decreasing weight, provide it in negative values
 * @param c1 The cognitive knowledge rate.
 * @param c2 The social knowledge rate.
 */
PSO::PSO(callback_t fitnessFunction, double s1, double s2, int p, int n, double w, double c1, double c2){
	this->fitnessFunction = fitnessFunction;
	this->s1 = s1;
	this->s2 = s2;
	this->p = p;
	this->n = n;
	this->c1 = c1;
	this->c2 = c2;
	if(w < 0){
		decreasingW = true;
		this->w = abs(w);
	}
	else{
		decreasingW = false;
		this->w = w;
	}

	Gb = new double[n];
	pb = new double*[p];
	pb_Fit = new double[p];
	x = new double*[p];
	fit = new double[p];
	v = new double*[p];
	for(int i=0; i < p; i++){
		pb[i] = new double[n];
		x[i] = new double[n];
		v[i] = new double[n];
	}
	seed = 1;
	nEvals = 0;
	Gb_Fit = DBL_MAX;
	gb = -1;
}

PSO::~PSO() {
	for(int i=0; i < p; i++){
		delete pb[i];
		delete x[i];
		delete v[i];
	}
	delete Gb;
	delete pb;
	delete pb_Fit;
	delete x;
	delete fit;
	delete v;
}

/**
 * @brief Startup the PSO.
 */
void PSO::startup(){
	seed = getRandomSeed();
	nEvals = 0;
	Gb_Fit = DBL_MAX;
	gb = -1;

	for(int j, i=0; i < p; i++){
		for(j=0; j < n; j++) v[i][j] = RAND_DOUBLE(seed, s1, s2)/10;
		for(j=0; j < n; j++) x[i][j] = RAND_DOUBLE(seed, s1, s2);
		COPY_ARR(x[i], pb[i], n);
		pb_Fit[i] = fit[i] = fitnessFunction(x[i], n);
		if(pb_Fit[i] < Gb_Fit){
			COPY_ARR(pb[i], Gb, n);
			Gb_Fit = pb_Fit[i];
			gb = i;
		}
	}
}

/**
 * @brief Obtain the next improvement.
 *
 * @param M The maximum number of evaluations allowed.
 */
void PSO::next(int M) {
	bool found = false;
	int i, j;
	double currW = w - (w / M) * nEvals;
	while(!found && nEvals < M){
		for(i=0; i < p; i++){
			for(j=0; j < n; j++){
				v[i][j] = currW * v[i][j] + c1 * RAND_DOUBLE(seed, 0, 1) * (pb[i][j] - x[i][j])
									  + c2 * RAND_DOUBLE(seed, 0, 1) * (Gb[j] - x[i][j]);
				x[i][j] += v[i][j];
				if(x[i][j] < s1)
					x[i][j] = s1;
				else if(x[i][j] > s2)
					x[i][j] = s2;
			}
			fit[i] = fitnessFunction(x[i], n);
			nEvals++;
		}
		for(i=0; i < p; i++){
			if(fit[i] < pb_Fit[i]){
				COPY_ARR(x[i], pb[i], n);
				pb_Fit[i] = fit[i];
				if(pb_Fit[i] < Gb_Fit){
					found = true;
					COPY_ARR(pb[i], Gb, n);
					Gb_Fit = pb_Fit[i];
					gb = i;
				}
			}
		}
		if(decreasingW) currW -= w / M;
	}
}

/**
 * @brief Get the best result obtained up to the moment (global best).
 *
 * @param _x A pointer to store the positions of the global best.
 * @return The index of the particle that found the global best position.
 */
int PSO::getBestPos(double *_x){
	COPY_ARR(pb[gb], _x, n);
	return gb;
}

/**
 * @brief Get the number of fitness function evaluations performed up to the moment.
 *
 * @return The number of evaluations performed.
 */
int PSO::getNEvals() {
	return nEvals;
}

/**
 * @brief Get the best fitness value found up to the moment.
 *
 *
 * @return The best fitness value found.
 */
double PSO::getFitness() {
	return Gb_Fit;
}

/**
 * @brief Get a random number to be used as seed for the random number generator.
 * 
 * This implementation can be adapted/changed as necessary.
 *
 * @return A random seed.
 */
unsigned int PSO::getRandomSeed(){
	struct timeval tv;
	gettimeofday(&tv,NULL);
	return tv.tv_usec;
}
