/*
 * Copyright (c) 2017, Peter Frank Perroni (pfperroni@gmail.com)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * Additionally, if used for any scientific purpose, the following reference
 * must be cited:
 * 
 * Peter Frank Perroni, Daniel Weingaertner, and Myriam Regattieri Delgado.
 * 2017. Estimating Stop Conditions of Swarm Based Stochastic Metaheuristic
 * Algorithms. In Proceedings of GECCO '17, Berlin, Germany, July 15-19, 2017,
 * pg 43-50. DOI: http://dx.doi.org/10.1145/3071178.3071326
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/**
 * @file
 * 
 * This file contains the wrapper code for Python calls.
 * 
 * @date 02/Jul/2017
 * @author Peter Frank Perroni (pfperroni@gmail.com)
 */

#include <stdio.h>
#include <string.h>
#include <iostream>
#include "PSO.hpp"
#include "CSMOn.hpp"

/**
 * @brief Contains a parameter received from / sent to the Python caller.
 */
typedef struct _Param {
	char *name;
	char c;
	int i;
	long l;
	float f;
	double d;
} Param;

/**
 * @brief The wrapper function for Python calls.
 * 
 * Call this method from Python using ctypes interface.
 *
 * @param method A string speficying the search method to be used.
 * @param inParam The parameters required to call the search method specified.
 * @param outParam The parameters returned by the search method called. This will be returned back to the Python caller.
 * @param outPos The final result containing the optimized position. This will be returned back to the Python caller.
 * @param fitnessFunction The callback Python function containing the code for the fitness function evaluation.
 */
extern "C" void search(char* method, Param *inParam, Param *outParam, double *outPos, callback_t fitnessFunction){

	// Replicate (and adapt) this block of code for each different search method you want to develop.
	// Begin template.
	if(strcmp(method, "pso") == 0){

		double w, c1, c2, s1, s2, R;
		int i, n, p, M;

		// Receive from Python caller any parameter you want.
		for(i=0; i < 9; i++){
			if(strcmp(inParam[i].name, "w") == 0) w = inParam[i].d;
			else if(strcmp(inParam[i].name, "c1") == 0) c1 = inParam[i].d;
			else if(strcmp(inParam[i].name, "c2") == 0) c2 = inParam[i].d;
			else if(strcmp(inParam[i].name, "n") == 0) n = inParam[i].i;
			else if(strcmp(inParam[i].name, "p") == 0) p = inParam[i].i;
			else if(strcmp(inParam[i].name, "s1") == 0) s1 = inParam[i].d;
			else if(strcmp(inParam[i].name, "s2") == 0) s2 = inParam[i].d;
			else if(strcmp(inParam[i].name, "M") == 0) M = inParam[i].i;
			else if(strcmp(inParam[i].name, "R") == 0) R = inParam[i].d;
		}
		cout << "[CSMOn] Called search method PSO with parameters: s1=" << s1 << ", s2=" << s2 << ", w=" << w
			 << ", c1=" << c1 << ", c2=" << c2 << ", n=" << n << ", p=" << p << ", M=" << M << ", R=" << R << endl;

		// Search method instantiation.
		PSO *pso = new PSO(fitnessFunction, s1, s2, p, n, w, c1, c2);

		// CSMOn instantiation and execution.
		CSMOn *csmon = new CSMOn(pso, M, R, 0);
		csmon->run();
		
		// Read the final results.
		double *bestPos = new double[n];
		csmon->getBestPos(bestPos);
		for(i=0; i < n; i++){ // Send the final result back to Python caller.
			outPos[i] = bestPos[i];
		}

		// Send any parameter you want back to Python caller.
		int evals = csmon->getNEvals();
		double fit = csmon->getFitness();
		for(i=0; i < 2; i++){
			if(strcmp(outParam[i].name, "nEvals") == 0) outParam[i].i = evals;
			else if(strcmp(outParam[i].name, "fit") == 0) outParam[i].d = fit;
		}

		// Remind to clear any object that is not owned by Python caller.
		delete bestPos;
		delete pso;
		delete csmon;
	}
	// End template.
	else printf("Search method %s not supported yet.", method);

}
