# Copyright (c) 2017, Peter Frank Perroni (pfperroni@gmail.com)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# Additionally, if used for any scientific purpose, the following reference
# must be cited:
# 
# Peter Frank Perroni, Daniel Weingaertner, and Myriam Regattieri Delgado.
# 2017. Estimating Stop Conditions of Swarm Based Stochastic Metaheuristic
# Algorithms. In Proceedings of GECCO '17, Berlin, Germany, July 15-19, 2017,
# pg 43-50. DOI: http://dx.doi.org/10.1145/3071178.3071326
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# CSMOn.cpp
#
# Created on: 02/Jul/2017
# Author: Peter Frank Perroni (pfperroni@gmail.com)
#
# This module contains all configuration required to call CSMOn C++ API.

import ctypes
from ctypes import *
from sys import version_info

# Send and receive parameters through this object.
class Param(ctypes.Structure):
    _fields_ = [('name', ctypes.c_char_p),
                ('c', ctypes.c_char),
                ('i', ctypes.c_int),
                ('l', ctypes.c_long),
                ('f', ctypes.c_float),
                ('d', ctypes.c_double)]

# List of parameters.
class ParamArrayType:
    def from_param(self, param):
        typename = type(param).__name__
        if hasattr(self, 'from_' + typename):
            return getattr(self, 'from_' + typename)(param)
        elif isinstance(param, ctypes.Array):
            return param
        else:
            raise TypeError("Can't convert %s" % typename)

    # Cast from array.array objects
    def from_array(self, param):
        if param.typecode != 'd':
            raise TypeError('must be an array of param')
        ptr, _ = param.buffer_info()
        return ctypes.cast(ptr, ctypes.POINTER(Param))

    # Cast from lists/tuples
    def from_list(self, param):
        val = ((Param)*len(param))(*param)
        return val

    from_tuple = from_list

    # Cast from a numpy array
    def from_ndarray(self, param):
        return param.ctypes.data_as(ctypes.POINTER(Param))

ParamArray = ParamArrayType()

try:
    from fitnessFunction import f
    callback_t = CFUNCTYPE(c_double, POINTER(c_double), c_int)
    fitFunc = callback_t(f)

    # Wrapper module must be called CSMOn_wrapper.so and must be present on current folder.
    _search = ctypes.CDLL("../CSMOn_wrapper.so").search
    def search(method, inParam, outParam):
        n = -1
        for it in inParam:
            if(it.name == "n"):
                n = it.i
                break
        if(n == -1):
            if version_info >= (3, 0):
                print("Dimension not provided!")
            else:
                print "Dimension not provided!"
            return NULL
        # Prepare the input parameters.
        pos = (c_double * n)()
        _outParam = (Param * len(outParam))(*outParam)
        _search.argtypes = (c_char_p, ParamArray, Param*len(_outParam), c_double * n, callback_t)

        # Call the C++ wrapper function.
        _search(method, inParam, _outParam, pos, fitFunc)

        # Read the output parameters.
        for i in range(_outParam._length_):
            outParam[i] = _outParam[i]

        # Return the optimized results.
        return list(pos)
        
except ImportError:
    if version_info >= (3, 0):
        print("Please define the function below:")
        print("   def f(x, n)")
        print("In a file called fitnessFunction.py, then reload this module.")
    else:
        print "Please define the function below:"
        print "   def f(x, n)"
        print "In a file called fitnessFunction.py, then reload this module."
